////////////////////////////////////////////////////////////////////////////////
// Main Class File:  FireworksApp.java
// File:             FireworksController.java
// Semester:         Spring 2008
//
// Author:           Tim Bahls
// CS Login:         bahls
// Lecturer:         NA
//////////////////////////// 80 columns wide ///////////////////////////////////

import java.awt.Graphics;

/**
 * This class controls the moving, drawing and creation of fireworks.
 * 
 * @author Tim
 * 
 */
public class FireworksController {

	private static final int NUMBER_OF_PARTICLES = 300;
	private static final double COLOR_VARIATION = .1;
	private static final double REMOVE_CHANCE = .01;

	// The list of Particles that we will use
	private ParticleList dotList;

	// the size of the window
	private int windowSize;

	// the code to pass into addParticleAtPoint()
	private int nextMode = 1;

	/**
	 * Create a new FireWorksController that starts with no fireworks.
	 * 
	 */
	public FireworksController(int size) {
		dotList = new ParticleList();
		this.windowSize = size;
	}

	/**
	 * Tell all the current Particles to move themselves. Also removes uneeded
	 * particles.
	 * 
	 */
	public void moveAll() {
		removeParticles();
		for (int i = 0; i < dotList.size(); i++) {
			dotList.get(i).takeOneStep();
		}
	}

	/**
	 * Tells all the current Particles to draw themselves using the given
	 * Graphics object.
	 * 
	 * @param graphics
	 *            The Graphics object to use for drawing
	 */
	public void paintAll(Graphics graphics) {
		for (int i = 0; i < dotList.size(); i++) {
			dotList.get(i).drawSelf(graphics);
		}
	}

	/**
	 * This method creates a new firework at the specified location. This is
	 * called by FireworksWindow any time the mouse is clicked on the window.
	 * The coordinates of the mouse click are passed in as parameters.
	 * 
	 * @param x
	 *            The x coordinate of the mouse click
	 * @param y
	 *            The y coordinate of the mouse click
	 */
	public void makeFirework(int x, int y) {
		double hue = Math.random();
		for (int i = 0; i < NUMBER_OF_PARTICLES; i++) {
			addParticleAtPoint(x, y, hue);
		}
	}

	/*
	 * This method adds a particle at the specified location with the specified
	 * color
	 */
	private void removeParticles() {
		for (int i = dotList.size() - 1; i > 0; i--) {
			if (Math.random() < REMOVE_CHANCE) {
				dotList.remove(i);
			}
		}
	}

	/*
	 * This method adds a particle at the specified location
	 */
	private void addParticleAtPoint(int x, int y, double hue) {
		double differentHue = hue + Math.random() * COLOR_VARIATION;
		Particle d;

		if (nextMode == 1) {
			d = new Particle(differentHue, x, y);
		} else if (nextMode == 2) {
			d = new GravityParticle(differentHue, x, y);
		} else if (nextMode == 3) {
			d = new SquareParticle(differentHue, x, y,
					(int) (Math.random() * 20) + 10);
		} else if (nextMode == 4) {
			d = new TeleportParticle(differentHue, x, y, windowSize, windowSize);
		} else if (nextMode == 5) {
			d = new BouncyParticle(hue, x, y, windowSize, windowSize);
		} else if (nextMode == 6) {
			d = new OrbitParticle(differentHue, x, y, windowSize / 2,
					windowSize / 2);
		} else if (nextMode == 7) {
			d = new LineParticle(differentHue, x, y, 4);
		} else {
			d = new FunkyParticle(differentHue, x, y);
		}

		dotList.add(d);
	}

	public int getSize() {
		return windowSize;
	}

	public void setMode(int newMode) {
		nextMode = newMode;
	}

}
