////////////////////////////////////////////////////////////////////////////////
// Main Class File:  FireworksApp.java
// File:             OrbitParticle.java
// Semester:         Spring 2008
//
// Author:           Tim Bahls
// CS Login:         bahls
// Lecturer:         self
//////////////////////////// 80 columns wide ///////////////////////////////////


/**
 * This particle Orbits around the location provided in the constructor.
 * 
 * @author Tim
 * 
 */
public class OrbitParticle extends Particle {

	// How strongly are we attracted to (xCenter, yCenter)?
	private static final double GRAVITY_STRENGTH = .5;
	// The x coordinate of where we are attracted
	private double xCenter;
	// The y coordinate of where we are attracted
	private double yCenter;

	/**
	 * Constructor that lets you determine color, start position, and center of
	 * the window.
	 * 
	 * @param hue
	 *            A number from 0 to 1 that specifies desired hue of the
	 *            Particle
	 * @param startX
	 *            The starting x coordinate
	 * @param startY
	 *            The starting y coordinate
	 * @param xCenter
	 *            The x coordinate of where we are attracted
	 * @param yCenter
	 *            The y coordinate of where we are attracted
	 */
	public OrbitParticle(double hue, double startX, double startY,
			double xCenter, double yCenter) {
		super(hue, startX, startY);
		this.xCenter = xCenter;
		this.yCenter = yCenter;
	}

	/**
	 * Move this Particle one frame's worth of movement. It uses some geometric
	 * math to give it "gravity" towards the center.
	 * 
	 */
	public void takeOneStep() {

		// calculate the vector to the center
		double xDistanceToCenter = xCenter - getXPosition();
		double yDistanceToCenter = yCenter - getYPosition();

		// Find the length of the vector
		double distanceToCenter = Math.sqrt(xDistanceToCenter
				* xDistanceToCenter + yDistanceToCenter * yDistanceToCenter);

		// Scale the difference vector to the length GRAVITY_STRENGTH
		double amountToAddToXVelocity = GRAVITY_STRENGTH * xDistanceToCenter
				/ distanceToCenter;
		double amountToAddToYVelocity = GRAVITY_STRENGTH * yDistanceToCenter
				/ distanceToCenter;

		//
		setXVelocity(getXVelocity() + amountToAddToXVelocity);
		setYVelocity(getYVelocity() + amountToAddToYVelocity);

		// actually move self
		super.takeOneStep();
	}

}
