////////////////////////////////////////////////////////////////////////////////
// Main Class File:  FireworksApp.java
// File:             Particle.java
// Semester:         Spring 2008
//
// Author:           Tim Bahls
// CS Login:         bahls
// Lecturer:         NA
//////////////////////////// 80 columns wide ///////////////////////////////////

import java.awt.Color;
import java.awt.Graphics;

/**
 * A Particle is a 2D colored dot that follows basic physics. It has a velocity
 * and is affected by gravity. It is able to move itself and to draw itself.
 * 
 * @author Tim
 * 
 */
public class Particle {

	// ////////////////////////////////
	// Constants
	// ////////////////////////////////

	// The diameter, in pixels, of each dot
	private static final int PARTICLE_DIAMETER = 7;

	// The fastest at which we can possible start
	private static final double MAXIMUM_SPEED = 7;

	// ////////////////////////////////
	// Instance fields
	// ////////////////////////////////

	// The x and y components of the position
	private double xPosition;
	private double yPosition;

	// The x and y components of the velocity
	private double xVelocity;
	private double yVelocity;

	// The number of times this particle has been moved
	private int timesMoved;

	// The color of the pixel
	private Color myColor;

	// ////////////////////////////////
	// Constructor
	// ////////////////////////////////

	/**
	 * Creates a new Particle based on the given parameters. You can specify the
	 * color and position. The velocity is randomly determined. The coordinates
	 * should be in java coordinates, so (0,0) is the upper lefthand corner
	 * <br />
	 * If you give 0 for the color, you get green. .2 gives orange, .3 gives
	 * pink, .4 gives magenta, .5 gives violet, .6 gives blue, .8 gives cyan,
	 * and 1 returns to green.
	 * 
	 * @param hue
	 *            A number from 0 to 1 that specifies desired hue of the
	 *            Particle
	 * @param startX
	 *            The starting x coordinate
	 * @param startY
	 *            The starting y coordinate
	 */

	public Particle(double hue, double startX, double startY) {
		timesMoved = 0;

		// generate speed and direction
		double angle = Math.random() * 360;
		double speed = Math.random() * MAXIMUM_SPEED;

		// set the position
		xPosition = startX;
		yPosition = startY;

		// get a bright color with the specified hue
		myColor = ColorOps.getBrightColor(hue);

		// Since the Math class requires radians, convert from degrees to
		// radians
		double angleInRadians = Math.toRadians(angle);

		// Conversion from polar to rectangular coordinates
		xVelocity = speed * Math.cos(angleInRadians);
		yVelocity = -speed * Math.sin(angleInRadians);
	}

	// ////////////////////////////////
	// public methods
	// ////////////////////////////////

	/**
	 * Draw this Particle onto the provided Graphics object
	 * 
	 * @param graphics
	 *            The Graphics object to draw onto.
	 */
	public void drawSelf(Graphics graphics) {
		// Set the color to the Color of this Particle
		graphics.setColor(myColor);

		// We want to center around our x and y coordinates, so we'll shift by
		// the radius
		int radius = PARTICLE_DIAMETER / 2;

		// Draw a solid oval of the correct size in the correct place. Notice we
		// need to cast the coordinates to ints
		graphics.fillOval((int) xPosition - radius, (int) yPosition - radius,
				PARTICLE_DIAMETER, PARTICLE_DIAMETER);

	}

	/**
	 * Move this Particle one frame's worth of movement
	 * 
	 */
	public void takeOneStep() {
		// Update the position according to the velocity
		xPosition += xVelocity;
		yPosition += yVelocity;

		// update timesMoved
		timesMoved++;
	}

	/**
	 * Return the number of times this Particle has been moved.
	 * 
	 * @return The number of times this particle has been moved
	 */
	public int getTimesMoved() {
		return timesMoved;
	}

	// //////////////////////
	// GETTERS
	// //////////////////////

	/**
	 * Return the X position of this Particle
	 * 
	 * @return The X position
	 */
	public double getXPosition() {
		return xPosition;
	}

	/**
	 * Return the Y position of this Particle
	 * 
	 * @return The Y position
	 */
	public double getYPosition() {
		return yPosition;
	}

	/**
	 * Return the X velocity of this Particle
	 * 
	 * @return The X velocity
	 */
	public double getXVelocity() {
		return xVelocity;
	}

	/**
	 * @return the yVelocity
	 */
	public double getYVelocity() {
		return yVelocity;
	}

	// //////////////////////
	// SETTERS
	// //////////////////////

	/**
	 * Set the x position
	 * 
	 * @param position
	 *            the xPosition to set
	 */
	public void setXPosition(double position) {
		xPosition = position;
	}

	/**
	 * Set the y position
	 * 
	 * @param position
	 *            the yPosition to set
	 */
	public void setYPosition(double position) {
		yPosition = position;
	}

	/**
	 * Set the x velocity
	 * 
	 * @param velocity
	 *            the xVelocity to set
	 */
	public void setXVelocity(double velocity) {
		xVelocity = velocity;
	}

	/**
	 * Sets the y velocity
	 * 
	 * @param velocity
	 *            the yVelocity to set
	 */
	public void setYVelocity(double velocity) {
		yVelocity = velocity;
	}

	/**
	 * Get the color of this particle
	 * @return the color of this particle
	 */
	public Color getColor(){
		return myColor;
	}
	
}
