////////////////////////////////////////////////////////////////////////////////
// Main Class File:  FireworksApp.java
// File:             ParticleList.java
// Semester:         Spring 2008
//
// Author:           Tim Bahls
// CS Login:         bahls
// Lecturer:         self
//////////////////////////// 80 columns wide ///////////////////////////////////

/**
 * A collection class that can only hold Particle objects. Particle can be added
 * or removed. They can be accessed and the number of particles can be
 * requested. Removing Particles is not garanteed to be stable--that is, the
 * order before and after may be different.
 */
public class ParticleList {

	// The underlying partially-fileld array of Particles
	private Particle[] dots;
	// The number of Particle objects this list holds.
	private int size;

	/**
	 * Create a new, initially empty, ParticleList
	 * 
	 */
	public ParticleList() {
		// start the array at a default of length 10.
		dots = new Particle[10];
		size = 0;
	}

	/**
	 * Get the Particle at the specified index. Returns null if the index is
	 * invalid.
	 * 
	 * @param index
	 *            The index of the desired Particle
	 * @return the desired Particle or null
	 */
	public Particle get(int index) {
		// if the index is valid
		if (isValidIndex(index)) {
			// return the requested element
			return dots[index];
		} else {
			// otherwise, return null
			return null;
		}
	}

	/**
	 * Add the given Particle to the list
	 * 
	 * @param toAdd
	 *            The Particle to add
	 */
	public void add(Particle toAdd) {
		// resize if we have no more room
		if (size == dots.length) {
			resize();
		}
		// add the Particle to the end of the array
		dots[size] = toAdd;
		size++;
	
	}

	
	/**
	 * Remove the Particle at the specified index. The other elements may or may
	 * not change position. The request is ignored if the index provided is not
	 * valid.
	 * 
	 * @param index
	 *            The index of the Particle to remove
	 */
	public void remove(int index) {
		// Ignore request if the index is invalid
		if (isValidIndex(index)) {
			// We copy over the element with the last element and reduce the
			// size
			dots[index] = dots[size - 1];
			size--;
		}
	}

	/**
	 * Returns true if the suppled index is valid for this ParticleList
	 * 
	 * @param index
	 *            The index to check
	 * @return true if the index is valid
	 */
	public boolean isValidIndex(int index) {
		return 0 <= index && index < size;
	}

	/**
	 * Returns true if the list has no Particles
	 * 
	 * @return true if the list has no Particles
	 */
	public boolean isEmpty() {
		return size() == 0;
	}

	/**
	 * Gets the number of Particles in this list.
	 * 
	 * @return The number of particles in this list
	 */
	public int size() {
		return size;
	}

	// Double the size of the underlying array
	private void resize() {
		// create a new array twice as big
		Particle[] tmp = new Particle[dots.length * 2];
		// copy in the elements
		for (int i = 0; i < dots.length; i++) {
			tmp[i] = dots[i];
		}
		dots = tmp;
	}
}
